from .util import commands, CommandHelp
from termcolor import cprint
import sys


def main():
    try:
        args = sys.argv[1:]
        process_command(args[0], args[1:])
    except IndexError:
        cprint('No command given', 'red')
        print()
        CommandHelp.execute()


def process_command(command=None, arguments=None):
    if command in commands.available_commands.keys():
        cprint(f'running {command}')
        commands.available_commands.get(command).execute(arguments)
    else:
        CommandHelp.execute(arguments)



from .command_build import CommandBuild
from .command_init import CommandInit
from .command_push import CommandPush
from .command_wipe import CommandWipe

available_commands = {
    'build': CommandBuild,
    'init': CommandInit,
    'push': CommandPush,
    'wipe': CommandWipe,
}



import os
import shutil

from termcolor import cprint

from .base import CommandBase


class CommandBuild(CommandBase):
    description = 'minify and attempt to compile pico application to bytecodes'

    @staticmethod
    def execute(arguments=None):
        cprint('copying codebase to build', 'blue')
        try:
            shutil.rmtree('build')
        except FileNotFoundError:
            pass
        os.mkdir('build')
        shutil.copytree('app', 'build/app')
        shutil.copyfile('main.py', 'build/main.py')
        shutil.copyfile('settings.py', 'build/settings.py')
        cprint('minifying codebase', 'blue')
        os.system('pyminify build --in-place')
        cprint('cross-compiling application libraries', 'blue')
        os.chdir('build')
        for root, dirs, files in os.walk("app", topdown=True):
            for name in files:
                original_name = os.path.join(root, name).replace('\\', '/')
                os.system(f'mpy-cross-v6 {original_name}')
                compiled_name = original_name.replace('.py', '.mpy')
                original_size = os.stat(original_name).st_size
                compiled_size = os.stat(compiled_name).st_size
                chosen_file = compiled_name if compiled_size < original_size else original_name
                saved_size = original_size - compiled_size if compiled_size < original_size else compiled_size - original_size
                print(f'{chosen_file} saving {saved_size} bytes')
                os.remove(compiled_name if original_name == chosen_file else original_name)
        os.chdir('..')



from termcolor import cprint
import os
import re
import subprocess

from .base import CommandBase


class CommandWipe(CommandBase):
    description = 'remove all files from a connected pico'

    @staticmethod
    def execute(arguments=None):
        cprint('wiping all code currently stored on device', 'red')

        def get_command_list(root=''):
            wipe_command_list = []

            output = subprocess.check_output(f"mpremote ls {root}", shell=True)

            for line in output.decode('utf-8').split(os.linesep):
                if not line or line.startswith('ls'):
                    continue
                line_path = re.search(r'\d+\s(.+)', line.strip()).group(1)
                if line_path.endswith('/'):
                    wipe_command_list += get_command_list(f'{root}{line_path}')
                wipe_command_list.append(f'mpremote rm {root}{line_path}')

            return wipe_command_list

        for wipe_command in get_command_list():
            os.system(wipe_command)

        os.system('mpremote reset')



from termcolor import cprint
from .commands import available_commands


class CommandHelp:
    description = 'show general help or specific command help'

    @staticmethod
    def execute(arguments=None):
        if arguments:
            name = arguments[0]
            try:
                command = available_commands.get(name)
                cprint(f'command: {name} - {command.description}', 'blue')
                if len(command.options) > 0:
                    print()
                    cprint('optional arguments:', 'blue')
                    cprint("\n".join(command.options))
            except AttributeError:
                cprint(f'command {name} does not exist', 'red')
                CommandHelp.execute()
        else:
            cprint('python pico-up', 'blue')
            print()
            for name in available_commands.keys():
                command = available_commands.get(name)
                cprint(f'{name.ljust(7)} {command.description}')
        print()



import os
import time
from termcolor import cprint
from .base import CommandBase
from .command_build import CommandBuild
from .command_wipe import CommandWipe


class CommandPush(CommandBase):
    description = 'push local application code to a connected pico'
    options = ['push build: run the build command and push built code to the pico']

    @staticmethod
    def execute(arguments=None):
        try:
            if arguments[0] == 'build':
                CommandBuild.execute()
                os.chdir('build')
        except IndexError:
            pass

        CommandWipe.execute()
        cprint('waiting for device', 'blue')
        time.sleep(2.0)
        cprint('pushing local code to device', 'blue')

        os.system(f'mpremote mkdir app')
        for root, dirs, files in os.walk("app", topdown=True):
            for name in files:
                remote_name = os.path.join(root, name).replace('\\', '/')
                os.system(f'mpremote cp {remote_name} :{remote_name}')
            for name in dirs:
                remote_name = os.path.join(root, name).replace('\\', '/')
                os.system(f'mpremote mkdir {remote_name}')

        os.system('mpremote cp main.py :main.py')
        os.system('mpremote cp settings.py :settings.py')
        os.system('mpremote reset')



import os
from termcolor import cprint
from .base import CommandBase


class CommandInit(CommandBase):
    description = 'initialise a python pico project in the current directory'

    @staticmethod
    def execute(arguments=None):
        cprint('creating a README file', 'blue')
        try:
            f = open("README.md", "x")
            f.write("# My Pico Project\n")
            f.close()
        except FileExistsError:
            cprint('README file already exists', 'red')

        cprint('creating a .gitignore', 'blue')
        try:
            f = open(".gitignore", "x")
            f.write("\n".join([
                '*.mpy',
                'build/',
            ]))
            f.close()
        except FileExistsError:
            cprint('.gitignore file already exists', 'red')

        cprint('creating settings file', 'blue')
        try:
            f = open("settings.py", "x")
            f.write("screen = 'unicorn'\n")
            f.close()
        except FileExistsError:
            cprint('settings file already exists', 'red')

        cprint('creating main entrypoint', 'blue')
        try:
            f = open("main.py", "x")
            f.write("print('Hello!')\n")
            f.close()
        except FileExistsError:
            cprint('main.py file already exists', 'red')

        cprint('creating application module', 'blue')
        try:
            os.mkdir('app')
        except FileExistsError:
            cprint('app directory already exists', 'red')

        try:
            f = open("app/__init__.py", "x")
            f.write("\n")
            f.close()
        except FileExistsError:
            cprint('app module file already exists', 'red')
